package com.displaynone.acss.components.auth.models.user.repository

import android.util.Log
import com.displaynone.acss.components.auth.models.AuthTokenPair
import com.displaynone.acss.config.Constants.serverUrl
import com.displaynone.acss.config.Network
import com.displaynone.acss.components.auth.models.user.repository.dto.RegisterUserDto
import com.displaynone.acss.components.auth.models.user.repository.dto.UserDTO
import com.displaynone.acss.components.auth.models.user.repository.dto.UserLoginDto
import io.ktor.client.call.body
import io.ktor.client.request.get
import io.ktor.client.request.headers
import io.ktor.client.request.post
import io.ktor.client.request.setBody
import io.ktor.client.statement.bodyAsText

import io.ktor.http.ContentType
import io.ktor.http.HttpHeaders
import io.ktor.http.HttpStatusCode
import io.ktor.http.contentType
import io.ktor.http.encodeURLPath
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import kotlinx.serialization.json.Json

class UserRepository(

) {
    suspend fun isUserExist(login: String): Result<Boolean> = withContext(Dispatchers.IO) {
        runCatching {
            val encodedLogin = login.encodeURLPath()
            val result = Network.client.get("$serverUrl/api/$encodedLogin/auth/")
            result.status != HttpStatusCode.OK
        }
    }
    suspend fun login(login: String, password: String): Result<AuthTokenPair> = withContext(Dispatchers.IO) {
        runCatching {
            val result = Network.client.post("$serverUrl/api/auth/login") {
                headers {
                    append(HttpHeaders.ContentType, ContentType.Application.Json.toString())
                }
                contentType(ContentType.Application.Json)
                setBody(UserLoginDto(login, password))
            }
            if (result.status != HttpStatusCode.OK) {
                error("Status ${result.status}: ${result.body<String>()}")
            }
//            val gson = Gson()
//            val tokenPair = gson.fromJson(result.bodyAsText(), AuthTokenPair::class.java)
            Log.d("UserRepository", result.bodyAsText())
//            result.body()
            val tokenPair = Json.decodeFromString<AuthTokenPair>(result.bodyAsText())
            Result.success(tokenPair)
        }.getOrElse { exception ->
            Result.failure(exception)
        }
    }
    suspend fun openDoor(token: String, code: String): Result<Int> = withContext(Dispatchers.IO){
        runCatching{
            val result = Network.client.post("$serverUrl/api/acs/open") {
                headers {
                    append(HttpHeaders.Authorization, "Bearer $token")
                }
                contentType(ContentType.Application.Json)
                setBody("""{ "code": $code }""")
            }
            if (result.status != HttpStatusCode.OK) {
                error("Status ${result.status}: ${result.body<String>()}")
            }
            Log.d("UserRepository", result.bodyAsText())
            result.status.value
        }
    }
    suspend fun getInfo(token: String): Result<UserDTO> = withContext(Dispatchers.IO){
        runCatching {
            val result = Network.client.get("$serverUrl/api/users/me") {
                headers {
                    append(HttpHeaders.Authorization, "Bearer $token")
                }
            }
            if (result.status != HttpStatusCode.OK) {
                error("Status ${result.status}: ${result.body<String>()}")
            }
            Log.d("UserRepository", result.bodyAsText())
            result.body()
        }
    }
    suspend fun getInfoByLogin(token: String, login: String): Result<UserDTO> = withContext(Dispatchers.IO){
        runCatching {
            val encodedLogin = login.encodeURLPath()
            val result = Network.client.get("$serverUrl/api/users/login/$encodedLogin") {
                headers {
                    append(HttpHeaders.Authorization, "Bearer $token")
                }
                setBody("""{ "code": "$encodedLogin" }""")
            }
            if (result.status != HttpStatusCode.OK) {
                error("Status ${result.status}: ${result.body<String>()}")
            }
            Log.d("UserRepository", result.bodyAsText())
            result.body()
        }
    }

    suspend fun register(login: String, password: String): Result<Unit> =
        withContext(Dispatchers.IO) {
            runCatching {
                val result = Network.client.post("$serverUrl/api/person/register") {
                    contentType(ContentType.Application.Json)
                    setBody(
                        RegisterUserDto(
                            login = login,
                            password = password,
                        )
                    )
                }
                if (result.status != HttpStatusCode.Created) {
                    Log.w("UserRepository", "Status: ${result.status}, Body: ${result.body<String>()}")
                    error("Status ${result.status}: ${result.body<String>()}")
                }
                Unit
            }
        }

}